use actix_files as fs;
use actix_web::{App, HttpServer, web};
use actix_web::middleware::Logger;
use tera::Tera;
use std::io;

mod config;
mod controllers;
mod app_middleware;
mod models;
mod routes;
mod utils;

// Import middleware components
use app_middleware::{RequestTimer, SecurityHeaders};

#[actix_web::main]
async fn main() -> io::Result<()> {
    // Initialize environment
    dotenv::dotenv().ok();
    env_logger::init_from_env(env_logger::Env::default().default_filter_or("info"));
    
    // Load configuration
    let config = config::get_config();
    let bind_address = format!("{}:{}", config.server.host, config.server.port);
    
    log::info!("Starting server at http://{}", bind_address);
    
    // Create and configure the HTTP server
    HttpServer::new(move || {
        // Initialize Tera templates
        let mut tera = match Tera::new(&format!("{}/**/*.html", config.templates.dir)) {
            Ok(t) => t,
            Err(e) => {
                log::error!("Parsing error(s): {}", e);
                ::std::process::exit(1);
            }
        };
        
        // Register custom Tera functions
        utils::register_tera_functions(&mut tera);
        
        App::new()
            // Enable logger middleware
            .wrap(Logger::default())
            // Add custom middleware
            .wrap(RequestTimer)
            .wrap(SecurityHeaders)
            // Configure static files
            .service(fs::Files::new("/static", "./src/static"))
            // Add Tera template engine
            .app_data(web::Data::new(tera))
            // Configure routes
            .configure(routes::configure_routes)
    })
    .bind(bind_address)?
    .workers(num_cpus::get())
    .run()
    .await
}
